@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
+use bytes::{buf::UninitSlice, BufMut};
 use core::mem::MaybeUninit;
 
 const BUFFER_LENGTH: usize = 64 - core::mem::size_of::<usize>();
@@ -19,43 +20,6 @@ impl SmallFixedBuffer {
             bytes: [MaybeUninit::<u8>::uninit(); BUFFER_LENGTH],
         }
     }
-    #[inline]
-    pub unsafe fn as_mut_slice(&mut self) -> &mut [u8] {
-        unsafe {
-            core::slice::from_raw_parts_mut(
-                (&raw mut self.bytes).cast::<u8>().add(self.idx),
-                BUFFER_LENGTH - self.idx,
-            )
-        }
-    }
-
-    #[inline]
-    pub unsafe fn set_written(&mut self, len: usize) {
-        debug_assert!(self.idx + len < BUFFER_LENGTH);
-        self.idx += len;
-    }
-
-    #[inline]
-    pub fn push(&mut self, value: u8) {
-        debug_assert!(self.idx + 1 < BUFFER_LENGTH);
-        unsafe {
-            core::ptr::write((&raw mut self.bytes).cast::<u8>().add(self.idx), value);
-            self.idx += 1;
-        };
-    }
-
-    #[inline]
-    pub fn extend_from_slice(&mut self, slice: &[u8]) {
-        debug_assert!(self.idx + slice.len() < BUFFER_LENGTH);
-        unsafe {
-            core::ptr::copy_nonoverlapping(
-                slice.as_ptr(),
-                (&raw mut self.bytes).cast::<u8>().add(self.idx),
-                slice.len(),
-            );
-            self.idx += slice.len();
-        }
-    }
 
     #[inline]
     pub fn as_ptr(&self) -> *const u8 {
@@ -67,3 +31,42 @@ impl SmallFixedBuffer {
         self.idx
     }
 }
+
+unsafe impl BufMut for SmallFixedBuffer {
+    #[inline]
+    unsafe fn advance_mut(&mut self, cnt: usize) {
+        self.idx += cnt;
+    }
+
+    #[inline]
+    fn chunk_mut(&mut self) -> &mut UninitSlice {
+        UninitSlice::uninit(&mut self.bytes)
+    }
+
+    #[inline]
+    fn remaining_mut(&self) -> usize {
+        BUFFER_LENGTH - self.idx
+    }
+
+    #[inline]
+    fn put_u8(&mut self, value: u8) {
+        debug_assert!(self.remaining_mut() > 1);
+        unsafe {
+            core::ptr::write((&raw mut self.bytes).cast::<u8>().add(self.idx), value);
+            self.advance_mut(1);
+        };
+    }
+
+    #[inline]
+    fn put_slice(&mut self, src: &[u8]) {
+        debug_assert!(self.remaining_mut() > src.len());
+        unsafe {
+            core::ptr::copy_nonoverlapping(
+                src.as_ptr(),
+                (&raw mut self.bytes).cast::<u8>().add(self.idx),
+                src.len(),
+            );
+            self.advance_mut(src.len());
+        }
+    }
+}
