@@ -1,12 +1,12 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
 use crate::serialize::error::SerializeError;
-use crate::str::{unicode_to_str, unicode_to_str_via_ffi};
+use crate::str::{PyStr, PyStrSubclass};
 
 use serde::ser::{Serialize, Serializer};
 
 #[repr(transparent)]
-pub struct StrSerializer {
+pub(crate) struct StrSerializer {
     ptr: *mut pyo3_ffi::PyObject,
 }
 
@@ -22,19 +22,15 @@ impl Serialize for StrSerializer {
     where
         S: Serializer,
     {
-        let uni = {
-            let tmp = unicode_to_str(self.ptr);
-            if unlikely!(tmp.is_none()) {
-                err!(SerializeError::InvalidStr)
-            }
-            tmp.unwrap()
-        };
-        serializer.serialize_str(uni)
+        match unsafe { PyStr::from_ptr_unchecked(self.ptr).to_str() } {
+            Some(uni) => serializer.serialize_str(uni),
+            None => err!(SerializeError::InvalidStr),
+        }
     }
 }
 
 #[repr(transparent)]
-pub struct StrSubclassSerializer {
+pub(crate) struct StrSubclassSerializer {
     ptr: *mut pyo3_ffi::PyObject,
 }
 
@@ -50,10 +46,9 @@ impl Serialize for StrSubclassSerializer {
     where
         S: Serializer,
     {
-        let uni = unicode_to_str_via_ffi(self.ptr);
-        if unlikely!(uni.is_none()) {
-            err!(SerializeError::InvalidStr)
+        match unsafe { PyStrSubclass::from_ptr_unchecked(self.ptr).to_str() } {
+            Some(uni) => serializer.serialize_str(uni),
+            None => err!(SerializeError::InvalidStr),
         }
-        serializer.serialize_str(uni.unwrap())
     }
 }
