@@ -1,5 +1,6 @@
 # SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
+import datetime
 import sys
 import uuid
 
@@ -7,6 +8,8 @@ import pytest
 
 import orjson
 
+from .util import numpy
+
 
 class Custom:
     def __init__(self):
@@ -141,7 +144,7 @@ class TestType:
             return str(obj)
 
         assert orjson.dumps({"a": ref}, default=default) == b'{"a":"%s"}' % str(
-            ref
+            ref,
         ).encode("utf-8")
 
     def test_default_func_list(self):
@@ -155,7 +158,7 @@ class TestType:
                 return [str(obj)]
 
         assert orjson.dumps({"a": ref}, default=default) == b'{"a":["%s"]}' % str(
-            ref
+            ref,
         ).encode("utf-8")
 
     def test_default_func_nested_list(self):
@@ -210,7 +213,7 @@ class TestType:
         """
         ref = Custom()
         assert orjson.dumps(ref, default=lambda x: str(x)) == b'"%s"' % str(ref).encode(
-            "utf-8"
+            "utf-8",
         )
 
     def test_default_callable_ok(self):
@@ -259,10 +262,24 @@ class TestType:
         def default(obj):
             return obj
 
+        refcount = sys.getrefcount(ref)
         with pytest.raises(orjson.JSONEncodeError):
             orjson.dumps(ref, default=default)
+        assert sys.getrefcount(ref) == refcount
 
-    def test_reference_cleanup_default(self):
+    def test_reference_cleanup_default_custom_pass(self):
+        ref = Custom()
+
+        def default(obj):
+            if isinstance(ref, Custom):
+                return str(ref)
+            raise TypeError
+
+        refcount = sys.getrefcount(ref)
+        orjson.dumps(ref, default=default)
+        assert sys.getrefcount(ref) == refcount
+
+    def test_reference_cleanup_default_custom_error(self):
         """
         references to encoded objects are cleaned up
         """
@@ -271,10 +288,44 @@ class TestType:
         def default(obj):
             raise TypeError
 
+        refcount = sys.getrefcount(ref)
         with pytest.raises(orjson.JSONEncodeError):
             orjson.dumps(ref, default=default)
+        assert sys.getrefcount(ref) == refcount
 
-        assert sys.getrefcount(ref) == 2  # one for ref, one for default
+    def test_reference_cleanup_default_subclass(self):
+        ref = datetime.datetime(1970, 1, 1, 0, 0, 0)
+
+        def default(obj):
+            if isinstance(ref, datetime.datetime):
+                return repr(ref)
+            raise TypeError
+
+        refcount = sys.getrefcount(ref)
+        orjson.dumps(ref, option=orjson.OPT_PASSTHROUGH_DATETIME, default=default)
+        assert sys.getrefcount(ref) == refcount
+
+    def test_reference_cleanup_default_subclass_lambda(self):
+        ref = uuid.uuid4()
+
+        refcount = sys.getrefcount(ref)
+        orjson.dumps(
+            ref,
+            option=orjson.OPT_PASSTHROUGH_DATETIME,
+            default=lambda val: str(val),
+        )
+        assert sys.getrefcount(ref) == refcount
+
+    @pytest.mark.skipif(numpy is None, reason="numpy is not installed")
+    def test_default_numpy(self):
+        ref = numpy.array([""] * 100)  # type: ignore
+        refcount = sys.getrefcount(ref)
+        orjson.dumps(
+            ref,
+            option=orjson.OPT_SERIALIZE_NUMPY,
+            default=lambda val: val.tolist(),
+        )
+        assert sys.getrefcount(ref) == refcount
 
     def test_default_set(self):
         """
